@@ -26,26 +26,23 @@
  along with OpenShot Library.  If not, see <http://www.gnu.org/licenses/>.
  """
 
-import os, math
+import os
+import uuid
+from operator import itemgetter
 from random import shuffle, randint, uniform
 
-from PyQt5.QtWidgets import *
+from PyQt5.QtWidgets import QDialog
 from PyQt5.QtGui import QIcon
 
-from classes import settings
-from classes import info, ui_util
+from classes import info, ui_util, time_parts
 from classes.logger import log
-from classes.query import Track, Clip, Transition
+from classes.query import Clip, Transition
 from classes.app import get_app
-from classes.metrics import *
+from classes.metrics import track_metric_screen
 from windows.views.add_to_timeline_treeview import TimelineTreeView
 
 import openshot
-
-try:
-    import json
-except ImportError:
-    import simplejson as json
+import json
 
 
 class AddToTimeline(QDialog):
@@ -53,7 +50,7 @@ class AddToTimeline(QDialog):
 
     ui_path = os.path.join(info.PATH, 'windows', 'ui', 'add-to-timeline.ui')
 
-    def btnMoveUpClicked(self, event):
+    def btnMoveUpClicked(self, checked):
         """Callback for move up button click"""
         log.info("btnMoveUpClicked")
 
@@ -65,7 +62,7 @@ class AddToTimeline(QDialog):
             selected_index = self.treeFiles.selected.row()
 
         # Ignore if empty files
-        if not files or selected_index == None:
+        if not files or selected_index is None:
             return
 
         # New index
@@ -82,7 +79,7 @@ class AddToTimeline(QDialog):
         idx = self.treeFiles.timeline_model.model.index(new_index, 0)
         self.treeFiles.setCurrentIndex(idx)
 
-    def btnMoveDownClicked(self, event):
+    def btnMoveDownClicked(self, checked):
         """Callback for move up button click"""
         log.info("btnMoveDownClicked")
 
@@ -94,7 +91,7 @@ class AddToTimeline(QDialog):
             selected_index = self.treeFiles.selected.row()
 
         # Ignore if empty files
-        if not files or selected_index == None:
+        if not files or selected_index is None:
             return
 
         # New index
@@ -111,17 +108,17 @@ class AddToTimeline(QDialog):
         idx = self.treeFiles.timeline_model.model.index(new_index, 0)
         self.treeFiles.setCurrentIndex(idx)
 
-    def btnShuffleClicked(self, event):
+    def btnShuffleClicked(self, checked):
         """Callback for move up button click"""
         log.info("btnShuffleClicked")
 
         # Shuffle files
-        files = shuffle(self.treeFiles.timeline_model.files)
+        shuffle(self.treeFiles.timeline_model.files)
 
         # Refresh tree
         self.treeFiles.refresh_view()
 
-    def btnRemoveClicked(self, event):
+    def btnRemoveClicked(self, checked):
         """Callback for move up button click"""
         log.info("btnRemoveClicked")
 
@@ -133,7 +130,7 @@ class AddToTimeline(QDialog):
             selected_index = self.treeFiles.selected.row()
 
         # Ignore if empty files
-        if not files or selected_index == None:
+        if not files or selected_index is None:
             return
 
         # Remove item
@@ -156,6 +153,10 @@ class AddToTimeline(QDialog):
         """ Ok button clicked """
         log.info('accept')
 
+        # Transaction id to group all updates together
+        tid = str(uuid.uuid4())
+        get_app().updates.transaction_id = tid
+
         # Get settings from form
         start_position = self.txtStartTime.value()
         track_num = self.cmbTrack.currentData()
@@ -174,7 +175,7 @@ class AddToTimeline(QDialog):
             random_transition = True
 
         # Get frames per second
-        fps = get_app().project.get(["fps"])
+        fps = get_app().project.get("fps")
         fps_float = float(fps["num"]) / float(fps["den"])
 
         # Loop through each file (in the current order)
@@ -183,15 +184,8 @@ class AddToTimeline(QDialog):
             clip = Clip()
             clip.data = {}
 
-            if (file.data["media_type"] == "video" or file.data["media_type"] == "image"):
-                # Determine thumb path
-                thumb_path = os.path.join(info.THUMBNAIL_PATH, "%s.png" % file.data["id"])
-            else:
-                # Audio file
-                thumb_path = os.path.join(info.PATH, "images", "AudioThumbnail.png")
-
             # Get file name
-            path, filename = os.path.split(file.data["path"])
+            filename = os.path.basename(file.data["path"])
 
             # Convert path to the correct relative path (based on this folder)
             file_path = file.absolute_path()
@@ -205,32 +199,21 @@ class AddToTimeline(QDialog):
             new_clip["layer"] = track_num
             new_clip["file_id"] = file.id
             new_clip["title"] = filename
-            new_clip["image"] = thumb_path
+            new_clip["reader"] = file.data
 
             # Skip any clips that are missing a 'reader' attribute
             # TODO: Determine why this even happens, as it shouldn't be possible
             if not new_clip.get("reader"):
                 continue  # Skip to next file
 
-            # Overwrite frame rate (incase the user changed it in the File Properties)
-            file_properties_fps = float(file.data["fps"]["num"]) / float(file.data["fps"]["den"])
-            file_fps = float(new_clip["reader"]["fps"]["num"]) / float(new_clip["reader"]["fps"]["den"])
-            fps_diff = file_fps / file_properties_fps
-            new_clip["reader"]["fps"]["num"] = file.data["fps"]["num"]
-            new_clip["reader"]["fps"]["den"] = file.data["fps"]["den"]
-            # Scale duration / length / and end properties
-            new_clip["reader"]["duration"] *= fps_diff
-            new_clip["end"] *= fps_diff
-            new_clip["duration"] *= fps_diff
-
             # Check for optional start and end attributes
             start_time = 0
             end_time = new_clip["reader"]["duration"]
 
-            if 'start' in file.data.keys():
+            if 'start' in file.data:
                 start_time = file.data['start']
                 new_clip["start"] = start_time
-            if 'end' in file.data.keys():
+            if 'end' in file.data:
                 end_time = file.data['end']
                 new_clip["end"] = end_time
 
@@ -239,32 +222,49 @@ class AddToTimeline(QDialog):
             if file.data["media_type"] == "image":
                 end_time = image_length
                 new_clip["end"] = end_time
+            else:
+                new_clip["end"] = end_time
 
             # Adjust Fade of Clips (if no transition is chosen)
             if not transition_path:
-                if fade_value != None:
+                if fade_value is not None:
                     # Overlap this clip with the previous one (if any)
                     position = max(start_position, new_clip["position"] - fade_length)
                     new_clip["position"] = position
 
-                if fade_value == 'Fade In' or fade_value == 'Fade In & Out':
+                if fade_value in ['Fade In', 'Fade In & Out']:
                     start = openshot.Point(round(start_time * fps_float) + 1, 0.0, openshot.BEZIER)
                     start_object = json.loads(start.Json())
-                    end = openshot.Point(min(round((start_time + fade_length) * fps_float) + 1, round(end_time * fps_float) + 1), 1.0, openshot.BEZIER)
+                    end = openshot.Point(
+                        min(
+                            round((start_time + fade_length) * fps_float) + 1,
+                            round(end_time * fps_float) + 1
+                            ),
+                        1.0,
+                        openshot.BEZIER)
                     end_object = json.loads(end.Json())
                     new_clip['alpha']["Points"].append(start_object)
                     new_clip['alpha']["Points"].append(end_object)
 
-                if fade_value == 'Fade Out' or fade_value == 'Fade In & Out':
-                    start = openshot.Point(max(round((end_time * fps_float) + 1) - (round(fade_length * fps_float) + 1), round(start_time * fps_float) + 1), 1.0, openshot.BEZIER)
+                if fade_value in ['Fade Out', 'Fade In & Out']:
+                    start = openshot.Point(
+                        max(
+                            round((end_time * fps_float) + 1) - (round(fade_length * fps_float) + 1),
+                            round(start_time * fps_float) + 1
+                            ),
+                        1.0,
+                        openshot.BEZIER)
                     start_object = json.loads(start.Json())
-                    end = openshot.Point(round(end_time * fps_float) + 1, 0.0, openshot.BEZIER)
+                    end = openshot.Point(
+                        round(end_time * fps_float) + 1,
+                        0.0,
+                        openshot.BEZIER)
                     end_object = json.loads(end.Json())
                     new_clip['alpha']["Points"].append(start_object)
                     new_clip['alpha']["Points"].append(end_object)
 
             # Adjust zoom amount
-            if zoom_value != None:
+            if zoom_value is not None:
                 # Location animation
                 if zoom_value == "Random":
                     animate_start_x = uniform(-0.5, 0.5)
@@ -334,7 +334,13 @@ class AddToTimeline(QDialog):
 
                 brightness = openshot.Keyframe()
                 brightness.AddPoint(1, 1.0, openshot.BEZIER)
-                brightness.AddPoint(round(min(transition_length, end_time - start_time) * fps_float) + 1, -1.0, openshot.BEZIER)
+                brightness.AddPoint(
+                    round(
+                        min(transition_length, end_time - start_time)
+                        * fps_float
+                        ) + 1,
+                    -1.0,
+                    openshot.BEZIER)
                 contrast = openshot.Keyframe(3.0)
 
                 # Create transition dictionary
@@ -360,7 +366,6 @@ class AddToTimeline(QDialog):
                 tran.data = transitions_data
                 tran.save()
 
-
             # Save Clip
             clip.data = new_clip
             clip.save()
@@ -368,6 +373,8 @@ class AddToTimeline(QDialog):
             # Increment position by length of clip
             position += (end_time - start_time)
 
+        # Clear transaction
+        get_app().updates.transaction_id = None
 
         # Accept dialog
         super(AddToTimeline, self).accept()
@@ -394,7 +401,7 @@ class AddToTimeline(QDialog):
                 # Don't subtract time from initial clip
                 if not transition_path:
                     # No transitions
-                    if fade_value != None:
+                    if fade_value is not None:
                         # Fade clip - subtract the fade length
                         duration -= fade_length
                 else:
@@ -405,34 +412,13 @@ class AddToTimeline(QDialog):
             total += duration
 
         # Get frames per second
-        fps = get_app().project.get(["fps"])
+        fps = get_app().project.get("fps")
 
         # Update label
-        total_parts = self.secondsToTime(total, fps["num"], fps["den"])
+        total_parts = time_parts.secondsToTime(total, fps["num"], fps["den"])
         timestamp = "%s:%s:%s:%s" % (total_parts["hour"], total_parts["min"], total_parts["sec"], total_parts["frame"])
         self.lblTotalLengthValue.setText(timestamp)
 
-    def padNumber(self, value, pad_length):
-        format_mask = '%%0%sd' % pad_length
-        return format_mask % value
-
-    def secondsToTime(self, secs, fps_num, fps_den):
-        # calculate time of playhead
-        milliseconds = secs * 1000
-        sec = math.floor(milliseconds/1000)
-        milli = milliseconds % 1000
-        min = math.floor(sec/60)
-        sec = sec % 60
-        hour = math.floor(min/60)
-        min = min % 60
-        day = math.floor(hour/24)
-        hour = hour % 24
-        week = math.floor(day/7)
-        day = day % 7
-
-        frame = round((milli / 1000.0) * (fps_num / fps_den)) + 1
-        return { "week":self.padNumber(week,2), "day":self.padNumber(day,2), "hour":self.padNumber(hour,2), "min":self.padNumber(min,2), "sec":self.padNumber(sec,2), "milli":self.padNumber(milli,2), "frame":self.padNumber(frame,2) };
-
     def reject(self):
         """ Cancel button clicked """
         log.info('reject')
@@ -450,13 +436,13 @@ class AddToTimeline(QDialog):
         # Init UI
         ui_util.init_ui(self)
 
-        # Get settings
-        self.settings = settings.get_settings()
-
         # Get translation object
         self.app = get_app()
         _ = self.app._tr
 
+        # Get settings
+        self.settings = self.app.get_settings()
+
         # Track metrics
         track_metric_screen("add-to-timeline-screen")
 
@@ -481,12 +467,14 @@ class AddToTimeline(QDialog):
         self.txtFadeLength.valueChanged.connect(self.updateTotal)
         self.txtTransitionLength.valueChanged.connect(self.updateTotal)
 
-        # Add all tracks to dropdown
-        tracks = Track.filter()
-        for track in reversed(tracks):
+        # Find display track number
+        all_tracks = get_app().project.get("layers")
+        display_count = len(all_tracks)
+        for track in reversed(sorted(all_tracks, key=itemgetter('number'))):
             # Add to dropdown
-            track_name = track.data['label'] or _("Track %s") % track.data['number']
-            self.cmbTrack.addItem(track_name, track.data['number'])
+            track_name = track.get('label') or _("Track %s") % display_count
+            self.cmbTrack.addItem(track_name, track.get('number'))
+            display_count -= 1
 
         # Add all fade options
         self.cmbFade.addItem(_('None'), None)
@@ -511,13 +499,12 @@ class AddToTimeline(QDialog):
         self.cmbTransition.addItem(_('Random'), 'random')
         self.transitions = []
         for group in transition_groups:
-            type = group["type"]
             dir = group["dir"]
             files = group["files"]
 
             for filename in sorted(files):
                 path = os.path.join(dir, filename)
-                (fileBaseName, fileExtension) = os.path.splitext(filename)
+                fileBaseName = os.path.splitext(filename)[0]
 
                 # Skip hidden files (such as .DS_Store, etc...)
                 if filename[0] == "." or "thumbs.db" in filename.lower():
